#!/usr/bin/env node
var __require = /* @__PURE__ */ ((x) => typeof require !== "undefined" ? require : typeof Proxy !== "undefined" ? new Proxy(x, {
  get: (a, b) => (typeof require !== "undefined" ? require : a)[b]
}) : x)(function(x) {
  if (typeof require !== "undefined")
    return require.apply(this, arguments);
  throw new Error('Dynamic require of "' + x + '" is not supported');
});

// node_modules/.pnpm/tsup@5.12.0_typescript@4.6.2/node_modules/tsup/assets/esm_shims.js
import { fileURLToPath } from "url";
import path from "path";
var getFilename = () => fileURLToPath(import.meta.url);
var getDirname = () => path.dirname(getFilename());
var __dirname = /* @__PURE__ */ getDirname();

// src/leetcode-dump.ts
import path6 from "path";
import chalk2 from "chalk";
import { program } from "commander";
import { config } from "dotenv";

// src/build.ts
import { execSync } from "child_process";
import path4 from "path";
import fs3 from "fs-extra";

// src/transform.ts
import fs2 from "fs";
import path3 from "path";

// src/constants.ts
var EXTS = {
  cpp: ".cpp",
  java: ".java",
  python: ".py",
  python3: ".py",
  c: ".c",
  csharp: ".cs",
  javascript: ".js",
  ruby: ".rb",
  swift: ".swift",
  golang: ".go",
  scala: ".scala",
  kotlin: ".kt",
  rust: ".rs",
  php: ".php",
  typescript: ".ts",
  racket: ".rkt",
  erlang: ".erl",
  elixir: ".ex"
};
var COMMENTS = {
  cpp: "//",
  java: "//",
  python: "#",
  python3: "#",
  c: "//",
  csharp: "//",
  javascript: "//",
  ruby: "#",
  swift: "//",
  golang: "//",
  scala: "//",
  kotlin: "//",
  rust: "//",
  php: "//",
  typescript: "//",
  racket: ";",
  erlang: "%",
  elixir: "#"
};
var LEETCODE_BASE = "https://leetcode.com";
var LEETCODE_DETAIL_BASE = LEETCODE_BASE + "/submissions/detail/";
var LEETCODE_SLUG_BASE = LEETCODE_BASE + "/problems/";
var README_TEMPLATE = `
# LeetCode Solutions

$card

## Solutions

| Problem | Difficulty | Tags | Solution |
| ------- | ---------- | ---- | -------- |
$table
`;
var DOC_TEMPLATE = `---
title: $title
---

# $title

$problem

$note

## Code

$code
`;

// src/utils.ts
import fs from "fs";
import path2 from "path";
import chalk from "chalk";
function get_package_meta() {
  return JSON.parse(fs.readFileSync(path2.join(__dirname, "../package.json"), "utf8"));
}
function readable_memory(mem) {
  const units = ["B", "KB", "MB", "GB", "TB"];
  let unit = 0;
  while (mem > 1e3) {
    mem /= 1e3;
    unit++;
  }
  return `${mem.toFixed(2)} ${units[unit]}`;
}
function sleep(ms) {
  return new Promise((resolve) => setTimeout(resolve, ms));
}
async function retry(func, max_retry2 = 3, cooldown = 1e3) {
  let error = null;
  for (let retry2 = 0; retry2 < max_retry2; retry2++) {
    try {
      return await func();
    } catch (err) {
      console.error(chalk.redBright(err));
      error = err;
      await sleep(cooldown);
    }
  }
  throw error;
}
var Logger = class {
  constructor(name = "", verbose = 3) {
    this.name = name;
    this.verbose = verbose;
    if (verbose) {
      this.log(`Verbose Mode: ${verbose}`);
    }
  }
  log(...args) {
    if (this.verbose >= 3) {
      console.log(chalk.gray(`[${this.name}]`), ...args);
    }
  }
  warn(...args) {
    if (this.verbose >= 2) {
      console.warn(chalk.yellowBright(`[${this.name}]`), ...args);
    }
  }
  error(...args) {
    if (this.verbose >= 1) {
      console.error(chalk.redBright(`[${this.name}]`), ...args);
    }
  }
};

// src/transform.ts
async function transform(source, output, verbose = false) {
  const logger = new Logger("Transform", verbose ? 3 : 0);
  source = path3.resolve(source);
  output = path3.resolve(output);
  logger.log(`Source: ${source}`);
  logger.log(`Output: ${output}`);
  if (!fs2.existsSync(source)) {
    throw new Error(`[Transform] ${source} does not exist`);
  }
  if (fs2.existsSync(output)) {
    throw new Error(`[Transform] ${output} already exists`);
  }
  try {
    fs2.mkdirSync(output, { recursive: true });
    fs2.writeFileSync(path3.resolve(output, "README.md"), fix(fs2.readFileSync(path3.resolve(source, "README.md"), "utf8")));
    logger.log("Root level README.md written");
    const problems = fs2.readdirSync(source).filter((name) => fs2.statSync(path3.resolve(source, name)).isDirectory());
    logger.log(`${problems.length} problems found`);
    for (const problem of problems) {
      const slug = problem.replace(/(\d+)\./, "$1").replace(/\s+/g, "-").replace("'", "").toLowerCase();
      fs2.mkdirSync(path3.resolve(output, slug), { recursive: true });
      const doc = gen_doc(path3.resolve(source, problem));
      fs2.writeFileSync(path3.resolve(output, slug, "index.md"), doc);
      logger.log(`${problem} written`);
    }
    logger.log("Transform Complete");
  } catch (err) {
    logger.error(err);
  }
}
function gen_doc(src) {
  const title = path3.basename(src);
  const problem = fs2.readFileSync(path3.resolve(src, "README.md"), "utf8");
  const note = fs2.readFileSync(path3.resolve(src, "NOTE.md"), "utf8");
  const sources = fs2.readdirSync(src).filter((name) => !name.endsWith(".md"));
  const solutions = sources.map((name) => {
    const solution = fs2.readFileSync(path3.resolve(src, name), "utf8");
    return { type: path3.extname(name).slice(1), solution };
  });
  return DOC_TEMPLATE.replace(/\$title/g, title).replace(/\$note/g, note).replace(/\$problem/g, problem.replace(/#[^\n]+\n/, "<br>\n\n")).replace(/\$code/g, solutions.map(({ type, solution }) => `### ${type.toUpperCase()}

\`\`\`${type}
${solution}
\`\`\``).join("\n\n"));
}
function fix(source) {
  return source.replace(/\(\.\/\d{1,4}\.[^)]+\)/g, (match) => match.replace(/(\d+)\./, "$1").replace(/(%20)+/g, "-").replace("'", "").toLowerCase()).replace(/([^.]\/)[^.]+\.([\w]+\))/g, "$1#$2");
}

// src/build.ts
async function build(source, output, config2, verbose = false) {
  const logger = new Logger("Build", verbose ? 3 : 0);
  source = path4.resolve(source);
  output = path4.resolve(output);
  logger.log(`Building site to ${output}`);
  if (!fs3.existsSync(source)) {
    throw new Error(`${source} does not exist`);
  }
  const workspace = fs3.mkdtempSync("site-tmp-");
  const tmp = path4.resolve(workspace, "source");
  fs3.symlinkSync(path4.resolve(__dirname, "../node_modules"), path4.resolve(workspace, "node_modules"));
  logger.log("Symlinked node_modules");
  transform(source, tmp, verbose);
  logger.log("Site transformed");
  if (fs3.existsSync(output)) {
    fs3.rmSync(output, { recursive: true });
  }
  await build_site(workspace, output, config2 ? JSON.parse(fs3.readFileSync(config2, "utf8")) : {
    plugins: [[__require.resolve("@vuepress/plugin-search"), {}]],
    title: "LeetCode Solutions"
  });
  fs3.rmSync(workspace, { recursive: true });
  logger.log("Build Complete");
}
async function build_site(workspace, dest, config2) {
  const exe = path4.resolve(__require.resolve("vuepress"), "../../bin/vuepress.js");
  fs3.mkdirSync(path4.resolve(workspace, "source", ".vuepress"), { recursive: true });
  fs3.writeFileSync(path4.resolve(workspace, "source", ".vuepress", "config.ts"), `export default ${JSON.stringify(config2, null, 4)};`);
  execSync(`${exe} build source --dest site`, { stdio: "inherit", cwd: workspace });
  fs3.moveSync(path4.resolve(workspace, "site"), dest, { overwrite: true });
}

// src/dump.ts
import fs4 from "fs";
import path5 from "path";
import { Credential, LeetCode } from "leetcode-query";
import fetch from "node-fetch";
import Ora from "ora";
import TurndownService from "turndown";
var turndown = new TurndownService().addRule("pre", {
  filter: "pre",
  replacement: (content, node) => "\n```\n" + node.textContent.replace("```", "\\`\\`\\`").trim() + "\n```\n"
}).addRule("sup", {
  filter: "sup",
  replacement: (content) => `^${content}`
});
var leetcode;
var max_retry = 3;
async function dump({
  session,
  output = path5.resolve("leetcode"),
  clean = true,
  cooldown = 250,
  timezone = "Asia/Taipei",
  pure = false,
  retry: retry2 = 3,
  verbose = true
}) {
  const { dir } = await setup({ timezone, retry: retry2, output, clean, session });
  const spinner = verbose ? Ora({ text: "Scanning...", spinner: "bouncingBar" }).start() : null;
  const { list, ac } = await get_list();
  spinner == null ? void 0 : spinner.succeed(`Scan Done. (${list.length} Problems, ${ac.length} Accepted)`);
  await sleep(cooldown);
  const table = [];
  for (let i = 0; i < ac.length; i++) {
    spinner == null ? void 0 : spinner.start(`Dumping Submissions... (${i + 1}/${ac.length})`);
    const { titleSlug: slug } = ac[i];
    const problem = await get_problem(slug);
    spinner && (spinner.text += ` ${problem.questionFrontendId}. ${problem.title}`);
    const folder = path5.resolve(dir, `${problem.questionFrontendId}. ${problem.title}`);
    if (!fs4.existsSync(folder)) {
      fs4.mkdirSync(folder, { recursive: true });
    }
    fs4.writeFileSync(path5.resolve(folder, "README.md"), `# ${problem.questionFrontendId}. ${problem.title}

Tags: ${problem.topicTags.map((t) => "`" + t.name + "`").join(", ")}

${turndown.turndown(problem.content)}`);
    fs4.writeFileSync(path5.resolve(folder, "NOTE.md"), problem.note);
    const submissions = await get_submissions(slug);
    spinner && (spinner.text += ` [${Object.keys(submissions).join(", ")}]`);
    await sleep(cooldown);
    const row = [
      `[${problem.questionFrontendId}. ${problem.title}](./${encodeURI(path5.basename(folder))}) [\u{1F517}](${LEETCODE_SLUG_BASE}${slug}/)`,
      problem.difficulty,
      problem.topicTags.map((t) => "`" + t.name + "`").join(", "),
      []
    ];
    for (const lang in submissions) {
      const ext = EXTS[lang];
      const file = path5.resolve(folder, `${slug}${ext}`);
      if (!fs4.existsSync(file)) {
        const submission = await get_submission(submissions[lang].id);
        const info = `${COMMENTS[lang]} ${problem.questionFrontendId}. ${problem.title} (${new Date(+submissions[lang].timestamp * 1e3).toLocaleDateString()})
${COMMENTS[lang]} Runtime: ${submission.runtime} ms (${submission.runtime_percentile.toFixed(2)}%) Memory: ${readable_memory(submission.memory)} (${submission.memory_percentile.toFixed(2)}%) 

`;
        fs4.writeFileSync(file, `${pure ? "" : info}${submission.code}`);
        row[3].push(`[${lang}](${encodeURI(`./${path5.basename(folder)}/${slug}${ext}`)})`);
        await sleep(cooldown);
      }
    }
    row[3].sort();
    table.push(row);
    spinner == null ? void 0 : spinner.succeed(`Dumped ${problem.questionFrontendId}. ${problem.title} [${Object.keys(submissions).join(", ")}]`);
  }
  spinner == null ? void 0 : spinner.succeed("Submissions Dumped.");
  fs4.writeFileSync(path5.resolve(dir, "README.md"), await create_toc(table));
}
async function setup({
  session,
  output,
  clean,
  timezone,
  retry: retry2
}) {
  process.env.TZ = timezone;
  max_retry = retry2;
  const dir = path5.resolve(output);
  if (fs4.existsSync(dir) && clean) {
    fs4.rmSync(dir, { recursive: true });
  }
  if (!fs4.existsSync(dir)) {
    fs4.mkdirSync(dir, { recursive: true });
  }
  const credential = new Credential();
  await credential.init(session);
  leetcode = new LeetCode(credential);
  return { dir };
}
async function get_list() {
  const ql = `query allQuestionsStatusesRaw { allQuestions: allQuestionsRaw { titleSlug questionFrontendId status } }`;
  const json = await retry(() => leetcode.graphql({
    query: ql,
    operationName: "allQuestionsStatusesRaw",
    variables: {}
  }), max_retry);
  const list = json.data.allQuestions;
  const ac = list.filter((item) => item.status === "ac").sort((a, b) => +a.questionFrontendId - +b.questionFrontendId);
  return { list, ac };
}
async function get_problem(slug) {
  const ql = `
    query questionData($slug: String!) {
        question(titleSlug: $slug) {
            questionFrontendId title content difficulty similarQuestions categoryTitle stats topicTags { name slug } note
        }
    }`;
  const json = (await retry(() => leetcode.graphql({ query: ql, operationName: "questionData", variables: { slug } }), max_retry)).data.question;
  json.similarQuestions = JSON.parse(json.similarQuestions);
  json.stats = JSON.parse(json.stats);
  return json;
}
async function get_submissions(slug) {
  const ql = `
    query Submissions($offset: Int!, $limit: Int!, $last: String, $slug: String!) {
        submissionList(offset: $offset, limit: $limit, lastKey: $last, questionSlug: $slug) {
            hasNext
            submissions { id statusDisplay lang runtime timestamp }
        }
    }`;
  const list = {};
  for (let i = 0; i < 10; i++) {
    const json = await retry(() => leetcode.graphql({
      query: ql,
      operationName: "Submissions",
      variables: { slug, offset: 0 * 20, limit: 20 }
    }), max_retry);
    const { submissions, hasNext } = json.data.submissionList;
    for (const submission of submissions) {
      const { id, statusDisplay, lang, runtime, timestamp } = submission;
      if (statusDisplay !== "Accepted") {
        continue;
      }
      if (!list[lang] || parseInt(runtime) < parseInt(list[lang].runtime) || parseInt(runtime) === parseInt(list[lang].runtime) && parseInt(timestamp) > parseInt(list[lang].timestamp)) {
        list[lang] = { id, timestamp, runtime };
      }
    }
    if (submissions.length === 0 || !hasNext) {
      break;
    }
  }
  return list;
}
async function get_submission(id) {
  return await retry(async () => {
    var _a;
    const res = await fetch(`${LEETCODE_DETAIL_BASE}${id}/`, {
      headers: {
        Cookie: `LEETCODE_SESSION=${leetcode.credential.session}`,
        Referer: LEETCODE_BASE
      }
    });
    const raw = await res.text();
    const data = (_a = raw.match(/var pageData = ({[^]+?});/)) == null ? void 0 : _a[1];
    const json = new Function("return " + data)();
    const result = {
      runtime: +json.runtime,
      runtime_distribution: json.runtimeDistributionFormatted ? JSON.parse(json.runtimeDistributionFormatted).distribution.map((item) => [+item[0], item[1]]) : null,
      runtime_percentile: 0,
      memory: +json.memory,
      memory_distribution: json.memoryDistributionFormatted ? JSON.parse(json.memoryDistributionFormatted).distribution.map((item) => [+item[0], item[1]]) : null,
      memory_percentile: 0,
      code: json.submissionCode
    };
    if (result.runtime_distribution) {
      result.runtime_percentile = result.runtime_distribution.reduce((acc, [usage, p]) => acc + (usage >= result.runtime ? p : 0), 0);
    }
    if (result.memory_distribution) {
      result.memory_percentile = result.memory_distribution.reduce((acc, [usage, p]) => acc + (usage >= result.memory / 1e3 ? p : 0), 0);
    }
    return result;
  }, max_retry);
}
async function create_toc(table) {
  const username = await get_username();
  const card = `[![LeetCode Stats Card](https://leetcode.card.workers.dev/?username=${username})](https://leetcode.com/${username}/)`;
  const table_str = table.map(([title, difficulty, tags, solutions]) => `| ${title} | ${difficulty} | ${tags} | ${solutions.join(" \\| ")} |`).join("\n");
  return README_TEMPLATE.replace("$card", card).replace("$table", table_str);
}
async function get_username() {
  return (await retry(() => leetcode.graphql({
    query: `query globalData { userStatus { username } }`,
    operationName: "globalData",
    variables: {}
  }), max_retry)).data.userStatus.username;
}

// src/leetcode-dump.ts
config();
var package_meta = get_package_meta();
program.version(chalk2`{cyanBright ${package_meta.name}} {yellowBright v${package_meta.version}}`);
program.option("-s, --session <session>", "Your LeetCode Session", process.env.LEETCODE_SESSION || "process.env.LEETCODE_SESSION").option("-o, --output <path>", "Output Dir", path6.resolve("leetcode")).option("-c, --clean", "Clean Output Dir Before Start", true).option("-cd, --cooldown <ms>", "Cooldown Between Actions, in ms", "200").option("-t, --timezone <timezone>", "Your Timezone", "Asia/Taipei").option("-p, --pure", "Pure Mode, No Additional Informations to Add", false).option("-r, --retry <times>", "Times to Retry When Fail", "3").option("-v, --verbose [bool]", "Verbose Mode", true).action(async () => {
  const opts = program.opts();
  await dump({
    session: opts.session,
    output: opts.output,
    clean: opts.clean,
    cooldown: +opts.cooldown,
    timezone: opts.timezone,
    pure: opts.pure,
    retry: +opts.retry,
    verbose: opts.verbose
  });
});
var build_command = program.command("build").description("Build static site from dumped solutions").option("-s, --source <path>", "Source Dir", path6.resolve("leetcode")).option("-o, --output <path>", "Output Dir", path6.resolve("site")).option("-c, --config <path>", "Vuepress Config Path", void 0).option("-v, --verbose [bool]", "Verbose Mode", true).action(() => {
  const opts = build_command.opts();
  build(opts.source, opts.output, opts.config, opts.verbose);
});
var transform_command = program.command("transform").description("Transform dumped solutions to a Vuepress source").option("-s, --source <path>", "Source Dir", path6.resolve("leetcode")).option("-o, --output <path>", "Output Dir", path6.resolve("site-source")).option("-v, --verbose [bool]", "Verbose Mode", true).action(() => {
  const opts = transform_command.opts();
  transform(opts.source, opts.output, opts.verbose);
});
program.parse();
